//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float4		g_SampleOffsets[ 15 ];
float		g_SampleWeights[ 15 ];
float3		g_vTexelSize			= float3( 1.0f / 256.0f, 1.0f / 256.0f, 0.0f );		// 3x3 blur
float		g_fAlphaMul = 1.5f;

texture		g_Texture;

sampler TextureSampler = sampler_state
{
	Texture		= < g_Texture >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

//-----------------------------------------------------------------------------
// Gaussian Blur
//-----------------------------------------------------------------------------
void PS_GaussianBlur( float2 i_TexCoord		: TEXCOORD0,
			          out float4 o_Color	: COLOR,
                      uniform bool a_bCheckAlpha )
{
    float4 vColor;
   // Accumulated color
	o_Color = float4( 0.0f, 0.0f, 0.0f, 1.0f );
   // Sample the taps (g_vSampleOffsets holds the texel offsets
   // and g_fSampleWeights holds the texel weights)  
   for(int i = 0; i < 15; i++ )
   {
      vColor = tex2D( TextureSampler, i_TexCoord + g_SampleOffsets[i] );
      if ( a_bCheckAlpha == true ) {
          vColor.rgb *= vColor.a;
      }
      
      o_Color.rgb += ( vColor.rgb * g_SampleWeights[i] );
   }  
   
   if ( a_bCheckAlpha == true ) {
        o_Color.rgb *= g_fAlphaMul;
   }
}

//-----------------------------------------------------------------------------
// 3x3 Blur
//-----------------------------------------------------------------------------
void PS_3x3Blur( float4 i_TexCoord			: TEXCOORD0,
				 out float4 o_Color			: COLOR,
                 uniform bool a_bCheckAlpha )
{
	float4 vTexCoords[9];
    float4 vColor;

	vTexCoords[0] = i_TexCoord;
	vTexCoords[1] = i_TexCoord + float4( -g_vTexelSize.x, 0.0f, 0.0f, 0.0f );
	vTexCoords[2] = i_TexCoord + float4(  g_vTexelSize.x, 0.0f, 0.0f, 0.0f );
	vTexCoords[3] = i_TexCoord + float4( 0.0f, -g_vTexelSize.y, 0.0f, 0.0f );
	vTexCoords[6] = i_TexCoord + float4( 0.0f,  g_vTexelSize.y, 0.0f, 0.0f );
	vTexCoords[4] = i_TexCoord + float4( -g_vTexelSize.x, -g_vTexelSize.y, 0.0f, 0.0f );
	vTexCoords[5] = i_TexCoord + float4(  g_vTexelSize.x, -g_vTexelSize.y, 0.0f, 0.0f );
	vTexCoords[7] = i_TexCoord + float4( -g_vTexelSize.x,  g_vTexelSize.y, 0.0f, 0.0f );
	vTexCoords[8] = i_TexCoord + float4(  g_vTexelSize.x,  g_vTexelSize.y, 0.0f, 0.0f );
	// Sample each of them checking whether the pixel under test is shadowed or not

	o_Color = 0.0f;
	for( int i = 0; i < 9; i++ )
	{
        vColor = tex2D( TextureSampler, vTexCoords[i] );
        if ( a_bCheckAlpha == true ) {
            vColor.rgb *= vColor.a;
        }        
		o_Color.rgb += vColor.rgb;
	}
	
	// Get the average
	o_Color /= 9.0f;
	
    o_Color.a = 1.0f;
}

//-----------------------------------------------------------------------------
// Technique
//-----------------------------------------------------------------------------
technique GaussianBlur
{
	pass P0
	{
		PixelShader		= compile ps_2_0 PS_GaussianBlur( false );
	}
}

technique GaussianBlurAlpha
{
	pass P0
	{
		PixelShader		= compile ps_2_0 PS_GaussianBlur( true );
	}
}

technique Blur_3x3
{
	pass P0
	{
		PixelShader		= compile ps_2_0 PS_3x3Blur( false );
	}
}

technique Blur_3x3Alpha
{
	pass P0
	{
		PixelShader		= compile ps_2_0 PS_3x3Blur( true );
	}
}


