//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------

//#define SKIN														// Compute position based on bones

#include "ComputeDepth.fx"

float4x4	g_mWorldViewProjection;									// World * View * Projection matrix
float		g_fCameraFarPlaneDist = 1500.0f;						// Distance to the far clip plane (the same as g_vFogParams.y)

#if SKIN
static const int MAX_MATRICES = 26;
float4x4	g_mWorldMatrixArray[ MAX_MATRICES ];
float4x4	g_mViewProj;
#define		NUM_OF_BONES 2
#endif

//-----------------------------------------------------------------------------
// Depth
//-----------------------------------------------------------------------------
void VS_Depth(	float4		i_Position		: POSITION,
#if SKIN
				float4		i_BlendWeights	: BLENDWEIGHT,
				float4		i_BlendIndices	: BLENDINDICES,
#endif
				out float4	o_Position		: POSITION,
				out float	o_Depth			: TEXCOORD0 )
{
#if !SKIN
	
	// Transform the position from object space to homogeneous projection space
	o_Position = mul( i_Position, g_mWorldViewProjection);
	
#else
	
	float4		vPositionWorld	= 0.0f;
	float		LastWeight		= 0.0f;
     
	// Cast the vectors to arrays for use in the for loop below
	float	BlendWeightsArray[4]	= (float[4])i_BlendWeights;  
	int		IndexArray[4]			= (int[4])i_BlendIndices;

	// Calculate the pos/normal using the "normal" weights and accumulate the weights to calculate the last weight
	for (int iBone = 0; iBone < NUM_OF_BONES-1; iBone++)
	{
		LastWeight = LastWeight + BlendWeightsArray[iBone];
		vPositionWorld += mul( i_Position, g_mWorldMatrixArray[IndexArray[iBone]] ) * BlendWeightsArray[iBone];
	}
	LastWeight = 1.0f - LastWeight; 

	// Now that we have the calculated weight, add in the final influence
	vPositionWorld += ( mul(i_Position, g_mWorldMatrixArray[IndexArray[NUM_OF_BONES-1]]) * LastWeight );

	// Transform position from world space into view and then projection space
	o_Position = mul( vPositionWorld, g_mViewProj );
#endif   	

	// Compute pixel depth
	o_Depth = o_Position.z / g_fCameraFarPlaneDist;
}

void PS_Depth(	float		i_Depth				: TEXCOORD0,
				out float4  o_Depth				: COLOR0 )
{
	o_Depth = ComputeDepth( i_Depth );
    o_Depth.a = 1.0f;
}

//-----------------------------------------------------------------------------
// Technique
//-----------------------------------------------------------------------------
technique Depth
{
	pass P0
	{
        VertexShader	= compile vs_1_1 VS_Depth();
		PixelShader		= compile ps_2_0 PS_Depth();
	}
}

