//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
//#define TITLE_BLUR

#define NUM_DOF_TAPS 12
float4      g_vFilterTaps[ NUM_DOF_TAPS ];
float		g_fMaxCoC				= 7.0f;

texture		g_Texture;
texture		g_DepthTexture;

#if TITLE_BLUR
texture		g_TitleTexture;

sampler TitleTextureSampler = sampler_state
{
	Texture		= < g_TitleTexture >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};
#endif

sampler TextureSampler = sampler_state
{
	Texture		= < g_Texture >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

sampler	DepthTextureSampler = sampler_state
{
	Texture		= < g_DepthTexture >;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

//-----------------------------------------------------------------------------
// RenderFinalImage
//-----------------------------------------------------------------------------
void PS_RenderFinalImage( float4 i_Texture			: TEXCOORD0,
						  out float4 o_Color		: COLOR )
{  
	float fMaxCoC = g_fMaxCoC;
	
	// Start with center sample color
	float4 colorSum = tex2D( TextureSampler, i_Texture);
	float totalContribution = 1.0f;
	
	// Depth and blurriness values for center sample
	float centerDepthBlur = tex2D( DepthTextureSampler, i_Texture ).r;
	
#if TITLE_BLUR
	// Sample title texture
	float4 vTitle = tex2D( TitleTextureSampler, i_Texture );

	fMaxCoC = ( ( vTitle.a != 0 ) ? 0.0f : g_fMaxCoC );
#endif
	
	// Compute CoC size based on blurriness
	float sizeCoC = centerDepthBlur * fMaxCoC;
	
	// Run through all filter taps
#if !TITLE_BLUR	
	for (int i = 0; i < NUM_DOF_TAPS; i++)
#else		
	for (int i = 0; i < NUM_DOF_TAPS - 1; i++)			// full loop is too complex for ps_2_0
#endif	
	{
		// Compute sample coordinates
		float4 tapCoord = i_Texture + ( g_vFilterTaps[i] * sizeCoC );
		
		// Fetch filter tap sample
		float4 tapColor = tex2D( TextureSampler, tapCoord );
		
		float tapDepthBlur = tex2D( DepthTextureSampler, tapCoord ).r;
		
		// Compute tap contribution based on depth and blurriness
		float tapContribution = ( ( tapDepthBlur > centerDepthBlur ) ? 1.0f : tapDepthBlur );
	
		// Accumulate color and sample contribution
		colorSum += tapColor * tapContribution;
		totalContribution += tapContribution;
	}
	
	// Normalize color sum
	o_Color = colorSum / totalContribution;

}

//-----------------------------------------------------------------------------
// Technique
//-----------------------------------------------------------------------------
technique RenderFinalImage
{
	pass P0
	{
		PixelShader		= compile ps_2_0 PS_RenderFinalImage();
	}
}
