//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------

//#define OUTPUT_DEPTH												// Output depth in second render target

float4x4	g_mWorldViewProjection;									// World * View * Projection matrix
float4x4	g_mWorld;												// World matrix for object
float3		g_vCameraPosition;                                      // Camera position in World Space

float4x4    g_mReflectedVectorRotation;

textureCUBE	g_EnvironmentTex;

#if OUTPUT_DEPTH
#include "ComputeDepth.fx"
float			g_fCameraFarPlaneDist = 1500.0f;					// Distance to the far clip plane (the same as g_vFogParams.y)
#endif

//-----------------------------------------------------------------------------
// Texture sampler
//-----------------------------------------------------------------------------
sampler g_EnvironmentTexSampler = sampler_state
{
	Texture		= < g_EnvironmentTex >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
	AddressV	= WRAP;
};

//-----------------------------------------------------------------------------
// EnvironmentMapping
//-----------------------------------------------------------------------------
void VS_EnvironmentMapping( float4		iPosition		: POSITION, 
                            float3		iNormal			: NORMAL,
                            out float4  oPosition		: POSITION,
                            out float3	oVertexToCamera	: TEXCOORD0,
#if !OUTPUT_DEPTH
                            out float3	oNormalWorld	: TEXCOORD1
#else
                            out float4	oNormalWorld	: TEXCOORD1
#endif
                            )
{
	// Transform the position from object space to homogeneous projection space
	oPosition = mul( iPosition, g_mWorldViewProjection);

	// Transform position and normal into world-space
	float4 vPositionWorld	= mul( iPosition, g_mWorld );
	oNormalWorld.xyz        = mul( iNormal, (float3x3)g_mWorld );
    
    // Compute vertex to camera vector
	oVertexToCamera = vPositionWorld - g_vCameraPosition;
    
#if OUTPUT_DEPTH
    // Compute pixel depth
	oNormalWorld.w = oPosition.z / g_fCameraFarPlaneDist;
#endif
}

void PS_EnvironmentMapping( float3      iVertexToCamera		: TEXCOORD0,
#if !OUTPUT_DEPTH
                            float3      iNormalWorld	        : TEXCOORD1,
#else
                            float4      iNormalWorld	        : TEXCOORD1,
#endif
                            out float4  oColor                  : COLOR0,
#if OUTPUT_DEPTH
					        out float4  o_Depth					: COLOR1,	// Output color (render target 1)
#endif
                            uniform bool uAnimate )
{
    // Normalize normal and vertex to camera vector
    float3 N = normalize( iNormalWorld );
	float3 I = normalize( iVertexToCamera );
    
    // Compute reflected vector
    float3 vReflected = reflect( I, N );
    
    // Animate reflected vector
    if ( uAnimate == true )
    {
        vReflected = mul( vReflected, g_mReflectedVectorRotation );
    }
    
    // Compute output color
    oColor = texCUBE( g_EnvironmentTexSampler, vReflected );
    oColor.a = 0.8f;

    // Add second animated texture to make better looking effect
    if ( uAnimate == true )
    {
        vReflected = mul( vReflected, g_mReflectedVectorRotation );
        oColor.rgb = ( oColor.rgb + texCUBE( g_EnvironmentTexSampler, vReflected ) ) * 0.5f; 
    }
    
#if OUTPUT_DEPTH
	o_Depth = ComputeDepth( iNormalWorld.w );
    o_Depth.a = 1.0f;
#endif    
}

//-----------------------------------------------------------------------------
// Technique
//-----------------------------------------------------------------------------
technique EnvironmentMapping
{
	pass P0
	{
        VertexShader	= compile vs_1_1 VS_EnvironmentMapping();
		PixelShader		= compile ps_2_0 PS_EnvironmentMapping( false );
	}
}

technique AnimatedEnvironmentMapping
{
	pass P0
	{
		COLORWRITEENABLE = RED | GREEN | BLUE;
		VertexShader	= compile vs_1_1 VS_EnvironmentMapping();
		PixelShader		= compile ps_2_0 PS_EnvironmentMapping( true );
	}
}
