//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float4x4	g_mWorldViewProjection;									// World * View * Projection matrix
float4		g_CameraPositionOS;

//-----------------------------------------------------------------------------
// HeatDistortion
//-----------------------------------------------------------------------------
void VS_HeatDistortion(	float4	i_Position		: POSITION, 
					 		float3	i_Normal		: NORMAL,
					 		out float4 o_Position	: POSITION,
							out float4 o_Strength	: TEXCOORD0 )
{

	// Transform the position from object space to homogeneous projection space
	o_Position = mul( i_Position, g_mWorldViewProjection);
	
	// Unpack normal
	i_Normal					= ( i_Normal - 0.5f ) * 2.0f;
	
	// Compute effect strength
	float fDistance 			= distance( g_CameraPositionOS, i_Position );
	float3 vCameraDirection 	= ( g_CameraPositionOS - i_Position ) /  fDistance;
	fDistance 					= 1.0f - ( fDistance / 200 );	
	o_Strength 					= pow( dot( vCameraDirection, i_Normal ), 2 ) * fDistance;
}

void PS_HeatDistortion(	float4 i_Strength		: TEXCOORD0,
							out float4 o_Color		: COLOR )
{
	o_Color = i_Strength;
}

//-----------------------------------------------------------------------------
// Techniques
//-----------------------------------------------------------------------------
technique HeatDistortion
{
	pass P0
	{
		VertexShader	= compile vs_1_1 VS_HeatDistortion();
		PixelShader		= compile ps_2_0 PS_HeatDistortion();
	}
}
