//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
//#define COMPRESSED_VECTORS										// Use compressed vectors

float4x4	g_mWorldViewProjection;		// World * View * Projection matrix
float4		g_EyePositionOS;
float		g_fElapsedTime;				// The time elapsed since the demo started

texture		g_HoloTexture;

#if !COMPRESSED_VECTORS
"ERROR - only compressed vectors"
#endif

#if COMPRESSED_VECTORS
float3 expand(float3 v)
{
    return (v-0.5)*2;
}
#endif

//-----------------------------------------------------------------------------
// Texture sampler
//-----------------------------------------------------------------------------
sampler HoloTextureSampler = sampler_state
{
	Texture		= < g_HoloTexture >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

///////////////////////////////////////////////////////////////////////////////

void VS_Holo(	float4 iPosition				: POSITION,
				float3 iNormal					: NORMAL,
				out float4 oPosition			: POSITION,
				out float2 oTextureCoord		: TEXCOORD0 )
{
#if COMPRESSED_VECTORS    
	// Unpack normal vector
	iNormal = expand( iNormal );    
#endif    
    
	// Transform the position from object space to homogeneous projection space
	oPosition = mul( iPosition, g_mWorldViewProjection );
   
    // Holo texture coordinates
    float3 vCameraDirection = normalize( g_EyePositionOS - iPosition );   
    oTextureCoord.x = dot( vCameraDirection, iNormal );
    
    // Add some noise to alpha value
    oTextureCoord.y = sin( g_fElapsedTime * iPosition.x * 0.1f ) * 0.4f;
}

void PS_Holo(	float2 iTexCoord				: TEXCOORD0,
                out float4 oColor				: COLOR )
{
    oColor = tex1D( HoloTextureSampler, iTexCoord.x );
    oColor.a += iTexCoord.y;
}

//**********************************************************************************************
// Techniques
//**********************************************************************************************
technique HoloShader
{
	pass P0
	{
		VertexShader	= compile vs_1_1 VS_Holo();
		PixelShader		= compile ps_2_0 PS_Holo();
    }
}