//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float		g_fFactor				= 1.0f;
bool		g_bPingDirection		= true;
float2		g_fScreenSize			= float2( 640, 480 );

texture		g_Texture0;
texture		g_Texture1;
texture		g_Texture2;
texture		g_Texture3;
texture		g_Texture4;

sampler TextureSampler0 = sampler_state
{
	Texture		= < g_Texture0 >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

sampler TextureSampler1 = sampler_state
{
	Texture		= < g_Texture1 >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

sampler TextureSampler2 = sampler_state
{
	Texture		= < g_Texture2 >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

sampler TextureSampler3 = sampler_state
{
	Texture		= < g_Texture3 >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

sampler TextureSampler4 = sampler_state
{
	Texture		= < g_Texture4 >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

//#define ADD_FRAME
//#define ADD_HORIZONTAL_SHIFT
//#define COLOR_SHIFT
//#define POWER_GLOW
//#define TITLE_BLUR

half3		g_vGlowFactor = half3( 0.8f, 1.0f, 1.0f );	// .x = glow texture multiply, .y = lerp value between normal and power glow, .z = lerp value between normal color and special glow effect

#if ADD_HORIZONTAL_SHIFT    
	#define NOISE_TEX_X_COORD 	0
	#define NOISE_STRENGTH		1
half2 	g_vNoiseData;
#endif

#if ADD_FRAME
half	g_fFrameScale = 1;
#endif

#if COLOR_SHIFT
half3	g_vColorShift = half3( 0.01, 0.015, 0.025 );
#endif

#if TITLE_BLUR
float4		g_vBlurTexelSize 	= float4( 1.0f / 256.0f, 1.0f / 128.0f, 1.0f / 256.0f * 2.0f, 1.0f / 128.0f * 2.0f );
#endif

#if DEMO_TITLE
float4	g_vDemoTitleData;		// .x = transparency, y. = effect strength, .z = noise texture coordinates, .w = noise strength
	#define DEMO_TITLE_TRANSPARENCY 		0
	#define DEMO_TITLE_STRENGTH				1
	#define DEMO_TITLE_NOISE_TEX_X_COORD 	2
	#define DEMO_TITLE_NOISE_STRENGTH		3

texture		g_NoiseTexture;

sampler 	NoiseTextureSampler = sampler_state
{
	Texture		= < g_NoiseTexture >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

#endif

//-----------------------------------------------------------------------------
// PS_MergeTextures_FinalHearth
//-----------------------------------------------------------------------------
void PS_MergeTextures_FinalHearth( float2 i_TexCoord	: TEXCOORD0,
			                       out float4 o_Color	: COLOR )
{
	float4 vBase = 0;
	float4 vGlow = 0;
	
#if ADD_FRAME	
	// Get frame color
	half2 vFrameTexCoord = ( ( i_TexCoord.xy - 0.5 ) * g_fFrameScale ) + 0.5;
	half3 fFrameColor = tex2D( TextureSampler3, vFrameTexCoord );
#endif

#if TITLE_BLUR
	// Sample title texture
	float4 vTitle = tex2D( TextureSampler4, i_TexCoord );
	float fTitlePixelNotPresent = 0.0f;
	if ( vTitle.a != 0 )
	{
		fTitlePixelNotPresent = 1.0f;
	}
#endif
    
#if ADD_HORIZONTAL_SHIFT    
	// Get noise
	half fNoise	= tex2D( TextureSampler2, float2( g_vNoiseData[ NOISE_TEX_X_COORD ], i_TexCoord.y ) ).r;
	
	// Modify texture coordinates
	i_TexCoord.x += ( fNoise - 0.5f ) * g_vNoiseData[ NOISE_STRENGTH ];
#endif
	
#if !COLOR_SHIFT
	// Get base and glow texels
	vBase	= tex2D( TextureSampler0, i_TexCoord );
	vGlow	= tex2D( TextureSampler1, i_TexCoord );
#else
	float2 TexCoord;

	TexCoord    = i_TexCoord;
	TexCoord.x  += g_vColorShift.x;
	vBase.r 	= tex2D( TextureSampler0, TexCoord ).r;
	vGlow.r	    = tex2D( TextureSampler1, TexCoord ).r;
	
	TexCoord    = i_TexCoord;
	TexCoord.x  += g_vColorShift.y;
	vBase.g 	= tex2D( TextureSampler0, TexCoord ).g;
	vGlow.g	    = tex2D( TextureSampler1, TexCoord ).g;    
	
	TexCoord    = i_TexCoord;
	TexCoord.x  += g_vColorShift.z;
	vBase.b 	= tex2D( TextureSampler0, TexCoord ).b;
	vGlow.b	    = tex2D( TextureSampler1, TexCoord ).b;
#endif
	
#if TITLE_BLUR
	float4	vBaseBlured = 0;
	float2	vTexCoords[9];

	vTexCoords[0] = i_TexCoord;
	vTexCoords[1] = i_TexCoord + float2( -g_vBlurTexelSize.x, 0.0f );
	vTexCoords[2] = i_TexCoord + float2(  g_vBlurTexelSize.x, 0.0f );
	vTexCoords[3] = i_TexCoord + float2( 0.0f, -g_vBlurTexelSize.y );
	vTexCoords[6] = i_TexCoord + float2( 0.0f,  g_vBlurTexelSize.y );
	vTexCoords[4] = i_TexCoord + float2( -g_vBlurTexelSize.z, -g_vBlurTexelSize.w );
	vTexCoords[5] = i_TexCoord + float2(  g_vBlurTexelSize.z, -g_vBlurTexelSize.w );
	vTexCoords[7] = i_TexCoord + float2( -g_vBlurTexelSize.z,  g_vBlurTexelSize.w );
	vTexCoords[8] = i_TexCoord + float2(  g_vBlurTexelSize.z,  g_vBlurTexelSize.w );
    
	// Sample each of them checking whether the pixel under test is shadowed or not
	for( int i = 0; i < 9; i++ )
	{
		vBaseBlured += tex2D( TextureSampler0, vTexCoords[ i ] );
	}
	
	vBaseBlured /= 9.0f;
	
	vTitle.rgb = lerp( vBaseBlured.rgb, vTitle.rgb, vTitle.a );
	
	if ( vTitle.a < 0.9 )
	{
		vTitle.rgb += vGlow.rgb;
	}
	
	vBase.rgb = lerp( vBase.rgb, vTitle.rgb, fTitlePixelNotPresent );
#endif
	
	// Compute final color
	vBase = vBase + vGlow * g_vGlowFactor.x;
	
#if POWER_GLOW
	// Compute color with power glow effect
	vGlow = lerp( vGlow * 2, vGlow * vGlow, g_vGlowFactor.y );
	o_Color = lerp( vBase, vGlow, g_vGlowFactor.z );
#if TITLE_BLUR
	o_Color.rgb = lerp( o_Color.rgb, vBase.rgb, fTitlePixelNotPresent );
#endif
    
#else
	o_Color = vBase;
#endif

#if DEMO_TITLE

	// Get noise
	half fDemoTitleNoise = tex2D( NoiseTextureSampler, float2( g_vDemoTitleData[ DEMO_TITLE_NOISE_TEX_X_COORD ], i_TexCoord.y ) ).r;
	   
	// Compute data for twirl effect
	float4 vTwirl = tex2D( TextureSampler4, i_TexCoord );
	vTwirl = ( vTwirl - 0.5f ) * g_vDemoTitleData[ DEMO_TITLE_STRENGTH ];
	float2 vDemoTitleTexCoord = i_TexCoord.xy + vTwirl.xy;
	
	// Modify texture coordinates
	vDemoTitleTexCoord.x += ( fDemoTitleNoise - 0.5f ) * g_vDemoTitleData[ DEMO_TITLE_NOISE_STRENGTH ];
	
	float4 vDemoTitleColor = tex2D( TextureSampler2, vDemoTitleTexCoord );
	o_Color.rgb = lerp( o_Color.rgb, vDemoTitleColor.rgb, vDemoTitleColor.a * g_vDemoTitleData[ DEMO_TITLE_TRANSPARENCY ] );
#endif    
   	
#if ADD_FRAME	
	// Blend with frame color
	o_Color.rgb *= fFrameColor;
#endif

#if BLEND_MULTIPLY || BLEND_SCREEN || BLEND_OVERLAY
	// Blending modes
	// http://www.simpelfilter.de/en/grundlagen/mixmods.html

//#define BLEND_MULTIPLY 1
//#define BLEND_SCREEN   1
//#define BLEND_OVERLAY 1

#ifndef ADD_FRAME
	half2 vFrameTexCoord = ( ( i_TexCoord.xy - 0.5 ) * g_fFrameScale ) + 0.5;
#endif

	half3 fMaskColor = tex2D( TextureSampler4, vFrameTexCoord );
 
#if BLEND_MULTIPLY
	o_Color.rgb *= fMaskColor;
#endif    

#if BLEND_SCREEN
	o_Color.rgb += fMaskColor;
#endif

#if BLEND_OVERLAY
	float3 c1 = 2 * o_Color.rgb * fMaskColor.rgb;
	float3 c2 = 1 - 2 * ( 1 - o_Color.rgb ) * ( 1 - fMaskColor.rgb );
	
	o_Color.r = ( o_Color.r <= 0.5 ) ? c1.r : c2.r;
	o_Color.g = ( o_Color.g <= 0.5 ) ? c1.g : c2.g;
	o_Color.b = ( o_Color.b <= 0.5 ) ? c1.b : c2.b;
#endif

#endif
}


//#define CREATE_NOISE

#if CREATE_NOISE
float4		g_vNoiseTexData = float4( 1.0f, 1.0f, 0.0f, 0.0f );		// .xy = Scale, zw = Texture shift
texture		g_NoiseTexture;

sampler NoiseTextureSampler = sampler_state
{
	Texture		= < g_NoiseTexture >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
	AddressV	= WRAP;
};

#endif

//-----------------------------------------------------------------------------
// MergeTextures_VeinPing
//-----------------------------------------------------------------------------
void VS_MergeTextures_VeinPing( float4 i_Position				: POSITION,			// the position of the vertex
								float2 i_TexCoord				: TEXCOORD0,		// the texture coordinates of the scene
								out float4 o_Position			: POSITION,
								out float2 o_TexCoord			: TEXCOORD0,
								out float2 o_PositionPS			: TEXCOORD1,
#if CREATE_NOISE
								out float2 o_NoiseTexCoord		: TEXCOORD2,
#endif
								uniform float2 u_TexCorrection )
{
	// Set position
	o_Position = i_Position;
	o_Position.w = 1.0f;
    
    // Set position for PixelShader
    o_PositionPS = i_Position.xy;
    
	// Set texture coordinates
    o_TexCoord = i_TexCoord;
    o_TexCoord.xy += u_TexCorrection;					// Make coorect mapping (avoid filtering) use this

#if CREATE_NOISE
	// Compute noise texture coordinates
	o_NoiseTexCoord = i_TexCoord * g_vNoiseTexData.xy;	// Scale
	o_NoiseTexCoord += g_vNoiseTexData.zw;				// Move
	o_NoiseTexCoord += u_TexCorrection;					// Make coorect mapping (avoid filtering) use this   	
#endif	
}

void PS_MergeTextures_VeinPing(	float2 i_TexCoord				: TEXCOORD0,
									float2 i_Position				: TEXCOORD1,
#if CREATE_NOISE
									float2 i_NoiseTexCoord			: TEXCOORD2,
#endif
									out float4 o_Color				: COLOR )
{
	// g_fFactor <-2,2>
	float fFactor = min( max( ( g_bPingDirection ? i_Position.x : 1.0f - i_Position.x ) + g_fFactor, 0.0f ), 1.0f );
	fFactor *= 2.0f;
    
	float4 vBase	= tex2D( TextureSampler0, i_TexCoord );
	float4 vGlow	= tex2D( TextureSampler1, i_TexCoord );
	
#if CREATE_NOISE	
	float4 vNoise	= tex2D( NoiseTextureSampler, i_NoiseTexCoord );    
	
	float4 vGlowAndNoise = vGlow * 2.0;
	vGlowAndNoise += ( ( vNoise - 0.5f ) * 0.1f );
	
	o_Color = lerp( vBase + vGlow, vGlowAndNoise, fFactor );
#else
	o_Color = vBase + vGlow;
#endif

#if ADD_FRAME
	// Get frame color
	half2 vFrameTexCoord = ( ( i_TexCoord.xy - 0.5 ) * g_fFrameScale ) + 0.5;
	half3 fFrameColor = tex2D( TextureSampler3, vFrameTexCoord );
	
	// Multiply by frame color
	o_Color.rgb *= fFrameColor;
#endif
}

//-----------------------------------------------------------------------------
// PS_MergeTextures2
//-----------------------------------------------------------------------------
void PS_MergeTextures2(	float2 i_TexCoord	: TEXCOORD0,
							out float4 o_Color	: COLOR )
{
	float4 vColor0 = tex2D( TextureSampler0, i_TexCoord );
	float4 vColor1 = tex2D( TextureSampler1, i_TexCoord );
	o_Color = lerp( vColor0, vColor1, g_fFactor );
}

//-----------------------------------------------------------------------------
// PS_MergeTextures5
//-----------------------------------------------------------------------------
void PS_MergeTextures5(	float2 i_TexCoord	: TEXCOORD0,
							out float4 o_Color	: COLOR )
{
	float4 vColor0 = tex2D( TextureSampler0, i_TexCoord );
	float4 vColor1 = tex2D( TextureSampler1, i_TexCoord );
	float4 vColor2 = tex2D( TextureSampler2, i_TexCoord );
	float4 vColor3 = tex2D( TextureSampler3, i_TexCoord );
	float4 vColor4 = tex2D( TextureSampler4, i_TexCoord );
	o_Color =  vColor0 + vColor1 + vColor2 + vColor3 + vColor4;
	o_Color *= g_fFactor;
	o_Color = o_Color * o_Color;  // power of 2 !
	}

//-----------------------------------------------------------------------------
// MergeTextures
//-----------------------------------------------------------------------------
technique MergeTextures2
{
	pass P0
	{
		PixelShader	= compile ps_2_0 PS_MergeTextures2();
	}
}

technique MergeTextures5
{
	pass P0
	{
		PixelShader	= compile ps_2_0 PS_MergeTextures5();
	}
}

technique MergeTextures_FinalHearth
{
	pass P0
	{
		PixelShader	= compile ps_2_0 PS_MergeTextures_FinalHearth();
	}
}

technique MergeTextures_VeinPing
{
	pass P0
	{
        CullMode		= CW;
        VertexShader= compile vs_1_1 VS_MergeTextures_VeinPing( 0.5f * ( 1.0f / g_fScreenSize ) );
		PixelShader	= compile ps_2_0 PS_MergeTextures_VeinPing();
	}
}
