//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------

//#define ENVIRONMENT												// Add environment mapping effect
//#define COMPRESSED_VECTORS										// Use compressed vectors

float4x4	g_WorldViewProjection;										// World * View * Projection matrix
float4		g_MaterialDiffuseColor	= float4( 1.0f, 1.0f, 1.0f, 1.0f );	// Material's diffuse color
float4		g_AmbientLightColor		= float4( 0.0f, 0.0f, 0.0f, 0.0f );	// Ambient light
float		g_fAlpha				= 1.0f;       

#if ENVIRONMENT
float4x4	g_mWorld;
float3		g_CameraPositionWS;
float		g_fEnvironmentStrength	= 0.7f;
textureCUBE	g_EnvironmentTexture;

sampler g_EnvironmentTexSampler = sampler_state
{
	Texture		= < g_EnvironmentTexture >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
	AddressV	= WRAP;
};
#endif

#if COMPRESSED_VECTORS
float3 expand(float3 v)
{
    return (v-0.5)*2;
}
#endif

#if !COMPRESSED_VECTORS
"ERROR - only compressed vectors"
#endif

//-----------------------------------------------------------------------------
// OnlyMaterialColor
//-----------------------------------------------------------------------------
void VS_OnlyMaterialColor( float4 i_Position	 : POSITION,
#if ENVIRONMENT
							float3		i_Normal	: NORMAL,
							out float3 o_NormalWS : TEXCOORD0,
							out float3 o_VertexToCamera : TEXCOORD1,
#endif
						   out float4 o_Position : POSITION )
{
	// Transform the position from object space to homogeneous projection space
	o_Position = mul( i_Position, g_WorldViewProjection );

#if ENVIRONMENT    
	
#if COMPRESSED_VECTORS
	// Unpack normal vector
	i_Normal = expand( i_Normal );    
#endif
	
	// Transform position and normal into world-space
	float4 vPositionWS	= mul( i_Position, g_mWorld );
	o_NormalWS	        = mul( i_Normal, (float3x3)g_mWorld );
    
    // Compute vertex to camera vector
	o_VertexToCamera = vPositionWS - g_CameraPositionWS;       
#endif    
}

void VS_OnlyMaterialColor_MakeBigger( float4 i_Position	: POSITION,
                                      float3 i_Normal		: NORMAL,
						              out float4 o_Position	: POSITION,
                                      uniform float a_fScale )
{
	// Transform the position from object space to homogeneous projection space
    i_Position.xyz = i_Position.xyz + i_Normal * a_fScale;
	o_Position = mul( i_Position, g_WorldViewProjection );
}

void PS_OnlyMaterialColor( 
#if ENVIRONMENT
							float3 i_NormalWS : TEXCOORD0,
							float3 i_VertexToCamera : TEXCOORD1,
#endif
							out float4 o_Color : COLOR )
{
	o_Color = g_MaterialDiffuseColor + g_AmbientLightColor;
	o_Color.a = g_fAlpha;
    
#if ENVIRONMENT
    // Compute reflected vector
    float3 vReflected = reflect( i_VertexToCamera, i_NormalWS ); 
    float3 vEnvColor = texCUBE( g_EnvironmentTexSampler, vReflected ).rgb;
    
    o_Color.rgb = lerp( o_Color.rgb, vEnvColor, g_fEnvironmentStrength );
#endif    
}

//-----------------------------------------------------------------------------
// Technique
//-----------------------------------------------------------------------------
technique OnlyMaterialColor
{
	pass P0
	{
		VertexShader	= compile vs_1_1 VS_OnlyMaterialColor();
#if ENVIRONMENT        
		PixelShader		= compile ps_2_0 PS_OnlyMaterialColor();
#else
		PixelShader		= compile ps_1_1 PS_OnlyMaterialColor();        
#endif
	}
}

technique OnlyMaterialColor_MakeBigger
{
	pass P0
	{
		VertexShader	= compile vs_1_1 VS_OnlyMaterialColor_MakeBigger( 7.0f );
#if ENVIRONMENT        
		PixelShader		= compile ps_2_0 PS_OnlyMaterialColor();
#else
		PixelShader		= compile ps_1_1 PS_OnlyMaterialColor();        
#endif
	}
}