//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
//#define ADD_NOISE_SHIFT 0
//#define CLAMP 0
//#defile CLAMP_UP 0
//#defile ANIM_TEX 0

float2		g_fTextureSize			= float2( 640, 480 );
float4		g_PositionScale			= float4( 0, 0, 1, 1 );	// .xy = position, .zw = scale
float4		g_Color					= float4( 1, 1, 1, 1 );
texture		g_Texture;

#if ADD_NOISE_SHIFT
	#define NOISE_TEX_X_COORD 	0
	#define NOISE_STRENGTH_X	1
	#define NOISE_STRENGTH_Y	2
half3 		g_vNoiseData;
texture		g_NoiseTexture;

sampler 	NoiseTextureSampler = sampler_state
{
	Texture		= < g_NoiseTexture >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};
#endif

#if CLAMP
float2		g_ClampRange = float2( 1, -1 );	// .x = upper Y, .y = lower Y
#endif

#if ANIM_TEX
float4		g_AnimTexPosScaleShift = float4( 0, 0, 1, 1 );	//.xy = position, .zw = scale
#endif

//-----------------------------------------------------------------------------
// Texture sampler
//-----------------------------------------------------------------------------
#if !ANIM_TEX
sampler TextureSampler = sampler_state
{
	Texture		= < g_Texture >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};
#else
sampler TextureSampler = sampler_state
{
	Texture		= < g_Texture >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	AddressU	= WRAP;
	AddressV	= WRAP;
};
#endif

//-----------------------------------------------------------------------------
// Name: VS_Overlay
//-----------------------------------------------------------------------------
void VS_Overlay(	float4			i_Position		: POSITION, 
					float2			i_TexCoord		: TEXCOORD0,
					out float4	 	o_Position		: POSITION,
#if !CLAMP
					out float2	 	o_TexCoord		: TEXCOORD0,
#else
					out float3	 	o_TexCoord		: TEXCOORD0,
#endif
					uniform float2 	u_TexCorrection )
{
	// Set position
	o_Position = i_Position;
	o_Position.xy *= g_PositionScale.zw;				// Scale
	o_Position.xy += g_PositionScale.xy;				// Move
    
	// Set texture coordinates
	o_TexCoord.xy = i_TexCoord;
	o_TexCoord.xy += u_TexCorrection;					// Make coorect mapping (avoid filtering) use this
	
#if ANIM_TEX
	o_TexCoord.xy *= g_AnimTexPosScaleShift.zw;
	o_TexCoord.xy += g_AnimTexPosScaleShift.xy;
#endif
	
#if CLAMP
	o_TexCoord.z = o_Position.y;
#endif

}

void PS_Overlay( 
#if !CLAMP
					float2		i_TexCoord			: TEXCOORD0,
#else
					float3		i_TexCoord			: TEXCOORD0,
#endif
					out float4	o_Color  			: COLOR )
{
#if ADD_NOISE_SHIFT    
	// Get noise
	float fNoise = tex2D( NoiseTextureSampler, float2( g_vNoiseData[ NOISE_TEX_X_COORD ], i_TexCoord.y ) ).r;
	
	// Modify texture coordinates
	fNoise = fNoise - 0.5f;
	i_TexCoord.x += fNoise * g_vNoiseData[ NOISE_STRENGTH_X ];
	i_TexCoord.y += fNoise * g_vNoiseData[ NOISE_STRENGTH_Y ];
#endif
	
	o_Color = tex2D( TextureSampler, i_TexCoord ) * g_Color;
	
#if CLAMP
#if CLAMP_UP
	if ( i_TexCoord.z > g_ClampRange.x )
	{
		o_Color = 0;
	}
#endif
	
	if ( i_TexCoord.z <= g_ClampRange.y )
	{
		o_Color = 0;
	}
#endif
}

//-----------------------------------------------------------------------------
// Name: Overlay
//-----------------------------------------------------------------------------
technique Overlay
{
	pass P0
	{
		CullMode		= CW;
		VertexShader	= compile vs_1_1 VS_Overlay( 0.5f * ( 1.0f / g_fTextureSize ) );
        PixelShader     = compile ps_2_0 PS_Overlay();
	}
}