//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float4x4 	g_mWorldViewProjection;

float3	g_vCameraRightVector;
float3 	g_vCameraUpVector;

texture 	g_DiffuseTexture;

sampler TextureSampler = sampler_state
{
	Texture		= < g_DiffuseTexture >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

//#define BLEND_ALPHA

//-----------------------------------------------------------------------------
// Particle
//-----------------------------------------------------------------------------
void VS_Particle(	float4 i_Position			: POSITION, 
					float4 i_Color				: COLOR0,
					float4 i_vTexCoordExtrude	: TEXCOORD0,
					float i_Rotation			: TEXCOORD1,
					out float4 o_Position		: POSITION,
					out float4 o_Color			: COLOR0,
					out float2 o_TexCoord		: TEXCOORD0 )
{
	// Compute rotation
	float fSin = sin( i_Rotation );
	float fCos = cos( i_Rotation );
	
	// Rotate
	float2 vLocalPosition;    
	vLocalPosition.x = i_vTexCoordExtrude.z * fCos - i_vTexCoordExtrude.w * fSin;
	vLocalPosition.y = i_vTexCoordExtrude.z * fSin + i_vTexCoordExtrude.w * fCos;
	    
	// Billboard vertices
	float4 vPositionBillboarded = i_Position;
	vPositionBillboarded.xyz += vLocalPosition.x * g_vCameraRightVector.xyz;
	vPositionBillboarded.xyz += vLocalPosition.y * g_vCameraUpVector.xyz;
	
	// Transform the position from object space to homogeneous projection space
	o_Position = mul( vPositionBillboarded, g_mWorldViewProjection);
	
	// Set color and texture coordinates
	o_Color = i_Color;
	o_TexCoord = i_vTexCoordExtrude.xy;
}

void PS_Particle(	float4 i_Color			: COLOR0,
					float2 i_TexCoord		: TEXCOORD0,
                    out float4 o_Color		: COLOR )
{
#if BLEND_ALPHA
	o_Color = tex2D( TextureSampler, i_TexCoord ) * i_Color;
#else
	o_Color.rgb = tex2D( TextureSampler, i_TexCoord ).rgb * i_Color.rgb;
	o_Color.a = i_Color.a;
#endif
}

//-----------------------------------------------------------------------------
// Techniques
//-----------------------------------------------------------------------------
technique Particle
{
	pass P0
	{
        COLORWRITEENABLE = RED | GREEN | BLUE;
		VertexShader	= compile vs_1_1 VS_Particle();
		PixelShader		= compile ps_1_1 PS_Particle();
	}
}