//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float2	g_ScreenSize		= float2( 640, 480 );
float4	g_vPositionSize		= float4( 0, 0, 100, 100 );	// .xy - position, .zw - size
float4	g_vTopColor			= float4( 1,0,0,1);
float4	g_vBottomColor		= float4( 0,1,0,1);
float2	g_AlphaColor		= float2( 0.7f, 0.4f );
float2	g_vClamp			= float2( 0,100);				// .x = bottom, .y = top - bottom

#define VALUE_BOTTOM 0
#define VALUE_RANGE  1

#define ALPHA_GRAPH 0
#define ALPHA_FRAME 1

//-----------------------------------------------------------------------------
// PerfCounter
//-----------------------------------------------------------------------------
void VS_PerfCounter(	float4 i_Position		: POSITION,
						out float4 o_Position	: POSITION,
						out float4 o_Color		: COLOR0,
						uniform float2 u_ScreenSize )
{  
	// Compute graph height <0-1>
	float vGraphHeight = i_Position.y;
	vGraphHeight = ( vGraphHeight - g_vClamp[ VALUE_BOTTOM ] ) / g_vClamp[ VALUE_RANGE ];
	vGraphHeight = clamp( vGraphHeight, 0.0f, 1.0f );
	
	// Interpoate between graph and frame (when i_Position.z is 1 then graph, if 0 then frame)
	vGraphHeight = lerp( i_Position.y, vGraphHeight, i_Position.z );
	
	// Basic position
	o_Position = float4( i_Position.x, vGraphHeight, 0.0f, 1.0f );
	o_Position.xy *= 2.0f;      // Screen is from -1 to 1
	
	// Scale graph
	o_Position.xy *= g_vPositionSize.zw / g_ScreenSize.xy; 
	
	// Move graph
	o_Position.xy -= 1.0f;
	o_Position.x += ( g_vPositionSize.x * 2 ) / g_ScreenSize.x;
	o_Position.y += ( ( ( g_ScreenSize.y - g_vPositionSize.w - g_vPositionSize.y ) * 2.0f) / g_ScreenSize.y );
	
	// Set graph color
	o_Color = lerp( g_vBottomColor, g_vTopColor, vGraphHeight );
    o_Color.a = g_AlphaColor[ ALPHA_GRAPH ];
	
	// Interpoate between graph and frame 
	o_Color = lerp( float4( 1.0f, 1.0f, 1.0f, g_AlphaColor[ ALPHA_FRAME ] ), o_Color, i_Position.z );
}

void PS_PerfCounter(	float4 i_Color		: COLOR0,
						out float4 o_Color	: COLOR )
{
	// Set color
	o_Color = i_Color;
}

//-----------------------------------------------------------------------------
// PerfCounter
//-----------------------------------------------------------------------------
technique PerfCounter
{
	pass P0
	{
		//AlphaBlendEnable = TRUE;
		VertexShader	= compile vs_1_1 VS_PerfCounter( g_ScreenSize );
		PixelShader	    = compile ps_1_1 PS_PerfCounter();
	}
}