//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float4x4	g_mWorldViewProjection;											// World * View * Projection matrix
float4x4	g_mWorldView;													// World * View
float4x4    g_mWorld;                                                       // World
float4x4    g_mShadowMapProjection;                                         // Shadow map projection matrix
float4x4    g_mShadowLightView;                                             // Shadow light/camera view matrix
float4		g_vCameraClipPlanes = float4( 0.1f, 1000.0f, 0.0f, 0.0f );		// Distance to the far clip plane

#define		SHADOW_INTENSITY	0.4f
#define		SHADOW_EPSILON		0.01f //0.00001f

texture		g_ShadowMapTexture;

sampler	ShadowMapSampler = sampler_state
{
	Texture		= < g_ShadowMapTexture >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

float ComputeDepth( float a_fPosWorldViewZ )
{
    return ( a_fPosWorldViewZ - g_vCameraClipPlanes.x ) / ( g_vCameraClipPlanes.y - g_vCameraClipPlanes.x );
}

//-----------------------------------------------------------------------------
// RenderDepth
//-----------------------------------------------------------------------------
void VS_RenderDepth( float4	i_Position	: POSITION,
					 out float4 o_Position	: POSITION,
					 out float4  o_Depth	: COLOR0 )
{
	// Output the transformed position
	o_Position = mul( i_Position, g_mWorldViewProjection );

	// Output the scene depth
    float4 vPosWorldView = mul( i_Position, g_mWorldView );
	o_Depth = ComputeDepth( vPosWorldView.z );                      // per pixel???
} 

void  PS_RenderDepth( float4		i_Depth : COLOR0,
					  out float4	o_Color : COLOR )
{
	// Output the scene depth
	o_Color = i_Depth;
    o_Color.a = 1.0f;
}

//-----------------------------------------------------------------------------
// VS_RenderShadow
//-----------------------------------------------------------------------------
void VS_RenderShadow( float4 i_Position			: POSITION, 
					  out float4 o_Position			: POSITION,
					  out float4 o_TextureShadow	: TEXCOORD0,
					  out float4 o_Depth			: TEXCOORD1 )
{
  	// Transform the position from object space to homogeneous projection space
	o_Position = mul( i_Position, g_mWorldViewProjection);

	// Convert object space position to world space and next convert to projective texture coordinates
	float4 vWorlPos = mul( i_Position, g_mWorld );
    
	o_TextureShadow = mul( vWorlPos, g_mShadowMapProjection );
	
	float4 vPosInShadowLight = mul( vWorlPos, g_mShadowLightView );
	o_Depth = ComputeDepth( vPosInShadowLight.z );                  // per pixel???
    
    o_Depth.xyzw = o_Depth;
}

void PS_RenderShadow( float4 i_TextureShadow	: TEXCOORD0,
					  float4 i_Depth			: TEXCOORD1,
					  out float4 o_Color		: COLOR )
{
	float fShadowTerm = 0.0f;
	float A = tex2Dproj( ShadowMapSampler, i_TextureShadow ).r;
	float B = i_Depth.x;//- SHADOW_EPSILON;

	// Texel is shadowed
	fShadowTerm = A < B ? SHADOW_INTENSITY : 1.0f;
    
    fShadowTerm = i_Depth.x <= 1.0f ? fShadowTerm : 1.0f;
    
	o_Color.rgb = fShadowTerm;
    o_Color.a = 1.0f;
}

//-----------------------------------------------------------------------------
// Techniques
//-----------------------------------------------------------------------------
technique RenderDepth
{
	pass P0
	{
		VertexShader	= compile vs_1_1 VS_RenderDepth();
		PixelShader		= compile ps_1_1 PS_RenderDepth();
	}
}

technique RenderShadow
{
	pass P0
	{
		VertexShader	= compile vs_1_1 VS_RenderShadow();
		PixelShader		= compile ps_2_0 PS_RenderShadow();
	}
}
