//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float4		g_vSinCos = float4( 0, 0, 0, 0 );   // .x = sin( time * 0.5f ), .y = cos( time * 0.5f ), .z = sin( -time * 0.5f ), .w = cos( -time * 0.5f );
float      g_fTransitionLevel = 0.0f;
float      g_fAlpha = 1.0f;
float2		g_fScreenSize			= float2( 640, 360 );

texture     g_SceneImageOld;
texture     g_SceneImageNew;
texture     g_NormalMap;

sampler SceneImageOldSampler = sampler_state
{
	Texture		= < g_SceneImageOld >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

sampler SceneImageNewSampler = sampler_state
{
	Texture		= < g_SceneImageNew >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

sampler NormalMapSampler = sampler_state
{
	Texture		= < g_NormalMap >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
	AddressV	= WRAP;
};

//-----------------------------------------------------------------------------
// Transition_Scene2To3
//-----------------------------------------------------------------------------
void VS_Transition_Scene2To3( float4 i_Position				: POSITION,			// the position of the vertex
                                float2 i_TexCoord				: TEXCOORD0,		// the texture coordinates of the scene
                                out float4 o_Position            : POSITION,
                                out float2 o_TexCoord            : TEXCOORD0,
                                out float2 o_BumpTexCoord0        : TEXCOORD1,
                                out float2 o_BumpTexCoord1        : TEXCOORD2,
                                uniform float2 u_TexCorrection )
{
	// Set position
	o_Position = i_Position;
	o_Position.w = 1.0f;
	
	// Set texture coordinates
    o_TexCoord = i_TexCoord;
    o_TexCoord.xy += u_TexCorrection;	//<- Make coorect mapping (avoid filtering) use this    
    
	// Compute noise texture coord
	float2 vWave = ( i_TexCoord - 0.5f ) * 2;
    float fSin = g_vSinCos.x;
	float fCos = g_vSinCos.y;
	o_BumpTexCoord0.x = vWave.x * fCos - vWave.y * fSin;
	o_BumpTexCoord0.y = vWave.x * fSin + vWave.y * fCos;    
    
    vWave = ( i_TexCoord - 0.5f ) * 2;
    fSin = g_vSinCos.z;
	fCos = g_vSinCos.w;
	o_BumpTexCoord1.x = vWave.x * fCos - vWave.y * fSin;
	o_BumpTexCoord1.y = vWave.x * fSin + vWave.y * fCos;  
}

void PS_Transition_Scene2To3( float2 i_TexCoord	: TEXCOORD0,
                float2 i_BumpTexCoord0         : TEXCOORD1,
                float2 i_BumpTexCoord1         : TEXCOORD2,
               out float4 o_Color	: COLOR )
{
	float3 vNormal = tex2D( NormalMapSampler, i_BumpTexCoord0 ).xyz + tex2D( NormalMapSampler, i_BumpTexCoord1 ).xyz;
    vNormal = vNormal - 0.5f;           // Expand vector = ( v - 0.5f ) * 2;  but Normal contains 2 vectors, so we have to divide it by 2 and finally we have v - 0.5f
    vNormal.xy *= g_fTransitionLevel;
    
    float4 vOldSceneColor = tex2D( SceneImageOldSampler, i_TexCoord.xy );
    float4 vNewSceneColor = tex2D( SceneImageNewSampler, i_TexCoord.xy + vNormal.xy );
    o_Color = lerp( vOldSceneColor, vNewSceneColor, g_fAlpha );
}

//-----------------------------------------------------------------------------
// MergeTextures
//-----------------------------------------------------------------------------
technique Transition_Scene2To3
{
	pass P0
	{
		CullMode		= CW;
        VertexShader	= compile vs_2_0 VS_Transition_Scene2To3( 0.5f * ( 1.0f / g_fScreenSize ) );
		PixelShader	    = compile ps_2_0 PS_Transition_Scene2To3();
	}
}