//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
//#define	ADD_NOISE
//#define	SCENE_TRANSITION
//#define	ADD_FRAME
//#define	ADD_GLOW

float2		g_fScreenSize		= float2( 640, 480 );

#if ADD_NOISE
float4		g_vNoiseTexData = float4( 1.0f, 1.0f, 0.0f, 0.0f );		// .xy = Scale, zw = Texture shift
#endif

#if ADD_FRAME
half	g_fFrameScale = 1;
#endif

#if SCENE_TRANSITION
#define		MASK_SMOOTH_SIZE    0.1f
float		g_fMaskFactor		= 0.0f;        // <-0.1 , 1.1>
#endif

#if ADD_NOISE
float		g_fNoiseStrength	= 0.1f;
#endif

//-----------------------------------------------------------------------------
// Textures
//-----------------------------------------------------------------------------
texture		g_TextureNewImage;
texture		g_TextureShadow;
texture		g_TextureHeatDistortion;
texture		g_TextureAnimDistortion;

#if SCENE_TRANSITION
texture		g_TextureOldImage;
texture		g_TextureMask;
#endif

#if ADD_NOISE
texture		g_TextureNoise;
#endif

#if	ADD_FRAME
texture		g_TextureFrame;
#endif

#if ADD_GLOW
texture		g_TextureGlow;
#endif

//-----------------------------------------------------------------------------
// Texture samplers
//-----------------------------------------------------------------------------
sampler NewImageSampler = sampler_state
{
	Texture		= < g_TextureNewImage >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

#if SCENE_TRANSITION
sampler OldImageSampler = sampler_state
{
	Texture		= < g_TextureOldImage >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};
#endif

sampler ShadowSampler = sampler_state
{
	Texture		= < g_TextureShadow >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

#if ADD_NOISE
sampler NoiseSampler = sampler_state
{
	Texture		= < g_TextureNoise >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
	AddressV	= WRAP;
};
#endif

#if SCENE_TRANSITION
sampler MaskSampler = sampler_state
{
	Texture		= < g_TextureMask >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};
#endif

sampler HeatDistortionSampler = sampler_state
{
	Texture		= < g_TextureHeatDistortion >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

sampler AnimDistortionSampler = sampler_state
{
	Texture		= < g_TextureAnimDistortion >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
	AddressV	= WRAP;
};

#if	ADD_FRAME
sampler FrameSampler = sampler_state
{
	Texture		= < g_TextureFrame >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};
#endif

#if ADD_GLOW
sampler GlowSampler = sampler_state
{
	Texture		= < g_TextureGlow >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};
#endif

//-----------------------------------------------------------------------------
// Transition
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// MergeTextures_VeinPing
//-----------------------------------------------------------------------------
void VS_Transition( 	float4 i_Position				: POSITION,			// the position of the vertex
						float2 i_TexCoord				: TEXCOORD0,		// the texture coordinates of the scene
						out float4 o_Position			: POSITION,
						out float2 o_TexCoord			: TEXCOORD0,
#if ADD_NOISE
						out float2 o_NoiseTexCoord		: TEXCOORD1,
#endif
						uniform float2 u_TexCorrection )
{
	// Set position
	o_Position = i_Position;
	o_Position.w = 1.0f;
	
	// Set texture coordinates
	o_TexCoord = i_TexCoord;
	o_TexCoord.xy += u_TexCorrection;					// Make coorect mapping (avoid filtering) use this
	
#if ADD_NOISE
	// Compute noise texture coordinates
	o_NoiseTexCoord = i_TexCoord * g_vNoiseTexData.xy;	// Scale
	o_NoiseTexCoord += g_vNoiseTexData.zw;				// Move
	o_NoiseTexCoord += u_TexCorrection;					// Make coorect mapping (avoid filtering) use this   	
#endif
}

void PS_Transition(	float2 i_TexCoord		: TEXCOORD0,
#if ADD_NOISE
						float2 i_NoiseTexCoord	: TEXCOORD1,
#endif
						out float4 o_Color		: COLOR )
{
	// Heat distortion
	half vHeatDistortionStrength = tex2D( HeatDistortionSampler, i_TexCoord ).r;
	
	// Get animated distortion vector
	float3 vAnimDistortion = tex2D( AnimDistortionSampler, i_TexCoord * float2( 3, 3 ) );
	vAnimDistortion = ( vAnimDistortion - 0.5f ) * 0.1f * vHeatDistortionStrength;
	
	i_TexCoord.x += vAnimDistortion.x;
	i_TexCoord.y += vAnimDistortion.y;
	
	float4 vSceneNewColor	= tex2D( NewImageSampler, i_TexCoord );		// Scene new image
#if 0
	float4 vShadowColor	    = tex2D( ShadowSampler, i_TexCoord );			// Shadow
#else
	
	float4 vShadowColor = 0;
	float2 vTexCoords[9];
    float4 vColor;
	
	float2 g_vTexelSize = float2( 1.0f / 256.0f, 1.0f / 256.0f );

	vTexCoords[0] = i_TexCoord;
	vTexCoords[1] = i_TexCoord + float2( -g_vTexelSize.x, 0.0f );
	vTexCoords[2] = i_TexCoord + float2(  g_vTexelSize.x, 0.0f );
	vTexCoords[3] = i_TexCoord + float2( 0.0f, -g_vTexelSize.y );
	vTexCoords[6] = i_TexCoord + float2( 0.0f,  g_vTexelSize.y );
	vTexCoords[4] = i_TexCoord + float2( -g_vTexelSize.x, -g_vTexelSize.y );
	vTexCoords[5] = i_TexCoord + float2(  g_vTexelSize.x, -g_vTexelSize.y );
	vTexCoords[7] = i_TexCoord + float2( -g_vTexelSize.x,  g_vTexelSize.y );
	vTexCoords[8] = i_TexCoord + float2(  g_vTexelSize.x,  g_vTexelSize.y );
    
	// Sample each of them checking whether the pixel under test is shadowed or not
	for( int i = 0; i < 9; i++ )
	{
        vShadowColor += tex2D( ShadowSampler, vTexCoords[ i ] );
	}
	
	// Get the average
	vShadowColor /= 9.0f;
	
#endif    
	
#if SCENE_TRANSITION	
	float4 vSceneOldColor	= tex2D( OldImageSampler, i_TexCoord );		// Scene old image
	float fMaskSampler = tex2D( MaskSampler, i_TexCoord ).r;				// Mask
	
	float fMergeFactor = ( fMaskSampler - g_fMaskFactor + MASK_SMOOTH_SIZE ) / ( 2 * MASK_SMOOTH_SIZE );
	fMergeFactor = clamp( fMergeFactor, 0.0f, 1.0f );
	
	// Compute final output color
	o_Color = lerp( vSceneNewColor, vSceneOldColor, fMergeFactor );
#else
	// Set final color
	o_Color = vSceneNewColor;
#endif
    
#if ADD_GLOW
	o_Color.rgb += tex2D( GlowSampler, i_TexCoord );
#endif    
    
    // Add shadow
	o_Color.rgb *= vShadowColor.rgb;
	
#if ADD_NOISE
	// Add noise 
	float4 vNoise	= tex2D( NoiseSampler, i_NoiseTexCoord );				// Noise
	o_Color.rgb 	+= ( ( vNoise.rgb - 0.5f ) * g_fNoiseStrength );		// noise = <-0.2 , 0.2>
#endif   

#if ADD_FRAME
	// Get frame color
	half2 vFrameTexCoord = ( ( i_TexCoord.xy - 0.5 ) * g_fFrameScale ) + 0.5;
	half3 fFrameColor = tex2D( FrameSampler, vFrameTexCoord );
	
	// Multiply by frame color
	o_Color.rgb *= fFrameColor;
#endif
}

//-----------------------------------------------------------------------------
// Technique
//-----------------------------------------------------------------------------
technique Transition
{
	pass P0
	{
		CullMode		= CW;
		VertexShader	= compile vs_1_1 VS_Transition( 0.5f * ( 1.0f / g_fScreenSize ) );
		PixelShader		= compile ps_2_0 PS_Transition();
	}
}