//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float4x4	g_mWorldViewProjection;						// World * View * Projection matrix
float		g_fTime                 = 0.0f;
float		g_fPower				= 1.0f;
float2		g_fScreenSize			= float2( 640, 480 );
float		g_ExtraColor			= 0.0f;

texture		g_Texture;

//-----------------------------------------------------------------------------
// Texture sampler
//-----------------------------------------------------------------------------
sampler g_TextureSampler = sampler_state
{
	Texture		= < g_Texture >;
	MinFilter	= LINEAR;  
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
};

//-----------------------------------------------------------------------------
// Name: VS_Warp
//-----------------------------------------------------------------------------
void VS_Warp( float4	     i_Position			: POSITION, 
              float4         i_TexCoord			: TEXCOORD0,
			  out float4	 o_Position			: POSITION,
			  out float4	 o_TexCoord			: TEXCOORD0,
			  uniform float2 u_TexCorrection )
{
	float2 vPosNormalized = float2( i_Position.x, i_Position.y );
    
    float fDist = (vPosNormalized.x*vPosNormalized.x) + (vPosNormalized.y*vPosNormalized.y);

    vPosNormalized.x *= ( 4.0f / 3.0f );
    vPosNormalized.y *= 0.7f;
    
    float a = vPosNormalized.x * vPosNormalized.x;
    float b = vPosNormalized.y * vPosNormalized.y;
    
    i_Position.z  = ( sin( a + g_fTime * 5 ) + sin(b + g_fTime * 5 ) );
    i_Position.z *= i_Position.z;
    i_Position.z *= 5;
    
    vPosNormalized.y *= 1.0f/0.45f;
    fDist = (vPosNormalized.x*vPosNormalized.x) + (vPosNormalized.y*vPosNormalized.y);
    
    i_Position.z = lerp( 0.0f, i_Position.z, g_fPower );
    
	// Transform the position from object space to homogeneous projection space
	o_Position = mul( i_Position, g_mWorldViewProjection);
    
	// Set texture coordinates
    o_TexCoord = i_TexCoord;
    o_TexCoord.xy += u_TexCorrection;	//<- Make coorect mapping (avoid filtering) use this    
}

void PS_Warp( float4       i_TexCoord       : TEXCOORD0,
			  out float4   o_Color  		: COLOR )
{
    o_Color = tex2D( g_TextureSampler, i_TexCoord );
    if ( o_Color.r > 0 )
    {
		o_Color.rgb += g_ExtraColor;		// Add extra color
    }
}

//-----------------------------------------------------------------------------
// Name: Warp
//-----------------------------------------------------------------------------
technique Warp
{
	pass P0
	{
		CullMode		= CW;
		VertexShader	= compile vs_1_1 VS_Warp( 0.5f * ( 1.0f / g_fScreenSize ) );
        PixelShader     = compile ps_2_0 PS_Warp();
	}
}